#!/bin/bash

# Lecteur de données Tempo Disc BlueMaestro
# Ce script est inspiré de celui de Elliot Larson disponible sur GitHub
# https://gist.github.com/elliotlarson/1e637da6613dbe3e777c

# Processus:
# 1. Arrêt et démarrage de HCI0
# 2. Démarrage de hcitool lescan
# 3. Démarrage de la lecture de données RAW avec hcidump
# 4. Réassemblage des données réparties sur plusieurs lignes
# 5. Vérification de l'entête du paquet de données.
#    - Si OK => Extraction des données
#            => STOP
#    - Si KO => On retourne et on continue

# Vérifie que l'on est en Root
if [[ $EUID -ne 0 ]];
then
  echo "Ce script doit être exécuté avec les privilèges administrateur"
  exit 1
fi

halt_hcitool_lescan() {
  sudo pkill --signal SIGINT hcitool
}

trap halt_hcitool_lescan INT

process_complete_packet() {
  # Exemple de packet :
  # > 04 3E 2B 02 01 00 01 C8 FC E0 F7 F1 F7 1F 02 01 06 11 FF 33 01 17 64 0E 10 00 10 00 C1 02 1F 00 66 01 00 09 09 46 37 46 31 46 37 45 30 B9
  
  local packet=${1//[\ |>]/}

  # Si le packet ne correspond pas à celui qu'on recherche, on retourne
  # Si il correspond, on extrait les données
  if [[ ! $packet =~ ^043E2B0201.{24}11FF ]]; then

    return

  else

  # Supprime le début du packet qui ne nous intéresse pas pour garder ce qui commence par 3301
  # 330117640E10001000C1021F0066010009094637463146374530B9
  packet=${packet:38:54}

  echo "Packet complet :" $packet
  echo

  battery_lvl=$((0x${packet:6:2}))
  time_interval=$((0x${packet:8:4}))
  stored_log_count=$((0x${packet:12:4}))
  current_temp=$((0x${packet:16:4}))
  current_temp=$(echo "scale=1; $current_temp/10" | bc)
  current_humidity=$((0x${packet:20:4}))
  current_humidity=$(echo "scale=1; $current_humidity/10" | bc)
  current_dew_point=$((0x${packet:24:4}))
  current_dew_point=$(echo "scale=1; $current_dew_point/10" | bc)
  mode=$((0x${packet:28:2}))
  breach_count=$((0x${packet:30:2}))
  name_identifier=$((0x${packet:32:2}))
  length_of_name=$((0x${packet:34:2}))
  name_of_device=$(echo ${packet:36:16} | xxd -r -p)

  echo "Battery (%) :" $battery_lvl
  echo "Time Interval (sec) :" $time_interval
  echo "Stored Log Cont :" $stored_log_count
  echo "Current Temp (°C) :" $current_temp
  echo "Current Humidity (%) :" $current_humidity
  echo "Current Dew Point (°C) :" $current_dew_point
  echo "Mode :" $mode
  echo "Breach Count :" $breach_count
  echo "Name Identifier :" $name_identifier
  echo "Length of Name :" $length_of_name
  echo "Name of Device :" $name_of_device

  exit 0

  fi
}

read_blescan_packet_dump() {
  # Les packets sont répartis sur plusieurs lignes et ont besoin d'être reconstitués
  # Exemple :
  # > 04 3E 2B 02 01 00 01 C8 FC E0 F7 F1 F7 1F 02 01 06 11 FF 33 
  #   01 17 64 0E 10 00 10 00 C1 02 1F 00 66 01 00 09 09 46 37 46 
  #   31 46 37 45 30 B9 
  packet=""
  while read line; do
    # Les packets qui débutent par : ">"
    if [[ $line =~ ^\> ]]; then
      # Traite le paquet terminé (sauf si c'est la première fois que c'est terminé)
      if [ "$packet" ]; then
        process_complete_packet "$packet"
      fi
      # Commencer un nouveau packet
      packet=$line
    else
      # Continuer à construire le packet
      packet="$packet $line"
    fi
  done
}

# On désactive le BT et on le réactive juste après pour éviter les erreurs
hciconfig hci0 down
hciconfig hci0 up

# Démarrage du scan BLE
# Si le périphérique BLE n'est pas trouvé dans les 10 sec on arrête lescan
# SIGINT (équivalent à ctrl-c)
timeout -s SIGINT 11s hcitool lescan --duplicates > /dev/null &
sleep 1

# On s'assure que le scan a démarré
if [ "$(pidof hcitool)" ]; then
  # On débute le scan des packets avec hcidump et on stream le tout à un process
   timeout -s SIGINT 10s hcidump --raw | read_blescan_packet_dump
else
  echo "ERREUR: Il semblerait que hcitool lescan n'a pas démarré correctement" >&2
  exit 1
fi
